var exec = require('child_process').exec;
var shellescape = require("shell-escape");
var ini = require("ini");
var fs = require('fs');

exports.timeDateCtlStatus = function(){
    return new Promise(function(resolve,reject){
        var parseTimeInfo = function(input){
            try{
                var output = {};
                input.split(/\n/).slice(0,-1).forEach(function(item){
                    var parsed =  /^\s*([\w\s]+): (.*)$/.exec(item);
                    output[parsed[1]] = parsed[2];
                });
                return output;
            }catch(e){
                reject(Error("unable-to-parse-time"));
                //$rootScope.$broadcast("console", {status:"err",message:"Unable to parse datetime info"});
            }
        };

        exec('timedatectl status',
            function (error, stdout, stderr) {
                if(error !== null){
                    reject(Error("unable-to-get-time"));
                    //socket.emit("datetime:error",{code:"unable-to-get-current-zone"});
                }else{
                    resolve(parseTimeInfo(stdout));
                    //socket.emit("datetime:timeInfoArrived",{info:stdout})
                }
            }
        )
    })
};

exports.getTimezonesList = function(){
    return new Promise(function(resolve,reject){
        let timeZones= fs.readFileSync('/usr/share/zoneinfo/zone.tab').toString().split('\n');
        let country= fs.readFileSync('/usr/share/zoneinfo/iso3166.tab').toString().split('\n');
        let resTimeZones = timeZones.map((line)=>{
            if (line.length < 3) return;
            if (line[0] == '#') return;
            let arr=line.split('\t');
            return {"code":arr[0],"tz":arr[2]}
        }).filter((line)=>{
            return line !== undefined;
        });
        let resCountry={};
        for(let i=0;i<country.length;i++){
            if(country[i].length<3||country[i][0]!='#'){
                let arr=country[i].split('\t');
                resCountry[arr[0]]=arr[1]
            }
        }
        let finalRes=[];
        for(let i=0;i<resTimeZones.length;i++){
            finalRes[i]={"code":resTimeZones[i].code,"tz":resTimeZones[i].tz,"text":resCountry[resTimeZones[i].code]+' '+resTimeZones[i].tz}
        }
        // console.log(finalRes);
        resolve(finalRes);
        /*console.time('readfile');
        let prom1=()=>{
            return new Promise((resolve,reject)=>{
                console.time('prom1');
                fs.readFile('/usr/share/zoneinfo/zone.tab',(err,data)=>{
                    console.timeEnd('prom1');
                    if(err) reject(err);
                    let lines=data.toString().split('\n');
                    let res = lines.map((line)=>{
                        if (line[0] == '#') return;
                        if (line.length < 3) return;
                        let arr=line.split('\t');
                        return {"code":arr[0],"tz":arr[2]}
                    }).filter((line)=>{
                        if(line!=undefined){
                            return line[0]!='#';
                        }else{
                            return false;
                        }

                    });
                    resolve(res);
                })
            });
        };

        let prom2=()=>{
            return new Promise((resolve,reject)=>{
                fs.readFile('/usr/share/zoneinfo/iso3166.tab',(err,data)=>{
                    if(err) reject(err);
                    let lines=data.toString().split('\n');
                    let res = lines.map((line)=>{
                        if(line[0]=='#')return;
                        if(line.length<3)return;
                        let arr=line.split('\t');
                        return {"code":arr[0],"country":arr[1]}
                    }).filter((line)=>{
                        if(line!=undefined){
                            return line[0]!='#';
                        }else{
                            return false;
                        }

                    });
                    resolve(res);
                })
            });
        };

        Promise.all([prom1(),prom2()]).then((results)=>{
            // console.log(results);
            console.timeEnd('readfile');
            // console.time('afterpromise');
            let finalRes=[];
            results[1].forEach((item)=>{
                results[0].forEach((item2)=>{
                    if(item.code==item2.code){
                        finalRes.push({'code':item2.code,'tz':item2.tz,'text':item.country+' '+item2.tz})
                    }
                })
            });
            resolve(finalRes);
            // console.timeEnd('afterpromise');
            // console.dir(finalRes);
        });
        console.time('exec');*/
        /*exec('timedatectl list-timezones',
            function (error, stdout, stderr) {
                if(error !== null){
                    reject(Error("unable-to-get-timezones"));
                }else{
                    try{
                        var output = stdout.split("\n").slice(0,-1);
                        // console.log('sssss')
                        console.log(output);
                        // console.timeEnd('exec');
                        // resolve(output)

                    }
                    catch(e){
                        // reject(Error("unable-to-parse-timezones"));
                    }
                }
            }
        )*/
    })
};

exports.setCurrentTimezone = function(data){
    return new Promise(function(resolve,reject){
        var escaped = shellescape(["timedatectl","set-timezone",data.zone]);
        exec(escaped,
            function (error, stdout, stderr) {
                if(error !== null){
                    reject(Error("unable-to-set-zone"));
                }else{
                    resolve();
                }
            }
        )
    })
};

exports.setDateTime = function(data){
    return new Promise(function(resolve,reject){
        if(data.ntpSync == "on"){
            exec("timedatectl set-ntp true",function(error){
                if(error !== null){
                    reject(Error("unable-to-set-time"));
                    //socket.emit("datetime:error",{code:"unable-to-set-time"});
                }else{
                    resolve();
                    //socket.emit("datetime:ok",{code:"set-time"});
                    //timeDateCtlStatus(socket);
                }
            })
        }else{
            exec("timedatectl set-ntp false",function(error){
                if(error !== null){
                    reject(Error("unable-to-set-time"));
                    //console.log(error);
                    //socket.emit("datetime:error",{code:"unable-to-set-time"});
                }else{
                    var escaped = shellescape(["timedatectl","set-time",data.date+" "+data.time]);
                    exec(escaped,
                        function (error, stdout, stderr) {
                            if(error !== null){
                                reject(Error("unable-to-set-time"));
                                //socket.emit("datetime:error",{code:"unable-to-set-time"});
                            }else{
                                resolve();
                                //socket.emit("datetime:ok",{code:"set-time"})
                            }
                        }
                    );
                }
            })
        }
    })
};

exports.getNTP = function(){
    return new Promise(function(resolve,reject){
        fs.readFile("/etc/systemd/timesyncd.conf",function(error,data){
            if(error !== null){
                reject(Error("unable-to-get-ntp"));
                //socket.emit("datetime:error",{code:"unable-to-get-ntp"});
            }else{
                try{
                    var result = ini.parse(data.toString())['Time']['FallbackNTP'].split(" ");
                    resolve({ntp:result});
                    //socket.emit("datetime:ntpArrived",{ntp:output});
                }catch(e){
                    reject(Error("unable-to-parse-ntp"));
                    //socket.emit("datetime:error",{code:"unable-to-get-ntp"});
                }
            }
        })
    })
};

exports.updateNTP = function(data){
    return new Promise(function(resolve,reject){
        try{
            var input = {
                "Time":{
                    "FallbackNTP":data.servers.join(" ")
                }
            };
            var serialized = ini.encode(input,{whitespace:true});
        }catch(e){
            reject (Error("unable-to-parse-ntp"));
        }
        fs.writeFile("/etc/systemd/timesyncd.conf", serialized, function(error){
            if(error){
                reject (Error("unable-to-set-ntp"));
                //socket.emit("datetime:error",{code:"unable-to-set-ntp"});
            }
            else{
                resolve();
                //socket.emit("datetime:ok",{code:"set-ntp"});
            }
        })
    })
};

exports.sysToHc = function(){
    return new Promise(function(resolve,reject){
        exec('hwclock --systohc -u',
            function (error, stdout, stderr) {
                if(error !== null){
                    reject();
                    //socket.emit("datetime:error",{code:"unable-to-sync"});
                }else{
                    resolve();
                    //socket.emit("datetime:ok",{code:"systohc"});
                    //timeDateCtlStatus();
                }
            }
        )
    });
};
