var exec = require('child_process').exec;
var fs = require('fs');
var ini = require("ini");
var shellescape = require("shell-escape");
var Netmask = require("netmask").Netmask;

exports.get = function(iface){
    return new Promise(function(resolve,reject){
        fs.readFile(`/etc/systemd/network/${iface}.network`,function(error,data){
            if(error !== null){
                //socket.emit("network:error",{code:"unable-to-get"});
                reject (Error("unable-to-get"));
            }else{
                try{
                    var output = ini.parse(data.toString());
                    resolve(output);
                    //socket.emit("network:dataArrived",output);
                }catch(e){
                    reject (Error("unable-to-parse"));
                    //socket.emit("network:error",{code:"unable-to-get"});
                }
            }
        })
    });
};

exports.set = function(data){
    const iface = data.iface;

    var dataToIni = function(data){
        try{
            var input = {
                "Match":{
                    "Name":iface
                }
            };
            if(data.mode === "dhcp"){
                input['Network'] = {"DHCP":"yes"};
                input['DHCP'] = {}
                if(data.buggy){
                    input['DHCP']["ClientIdentifier"] = "mac";
                }
                if(data.metric){
                    input['DHCP']["RouteMetric"] = data.metric
                }
            }else if(data.mode === "static"){
                input['Network'] = {
                    "Address": data.ip4,
                };
                if(data.metric){
                    input['Route'] = {
                        'Metric': data.metric
                    }
                }
                if(data.gateway){
                    input['Network']["Gateway"] = data.gateway
                }
                if(typeof data.dns == "object"){
                    var dns = data.dns.filter(function(item){
                        return item != "";
                    });
                    if(dns.length > 0){
                        input['Network']['DNS'] = data.dns.join(", ")
                    }
                }
            }
            return ini.encode(input,{whitespace:true});
        }catch(e){
            socket.emit("network:error",{code:"unable-to-set"});
            return false;
        }
    };

    var writeNetworkInfo = function(serialized){
        return new Promise(function(resolve, reject){
            fs.writeFile(`/etc/systemd/network/${iface}.network`, serialized, function(error){
                if(error){
                    reject(Error("unable-to-set"));
                    //socket.emit("network:error",{code:"unable-to-set"});
                }
                else{
                    resolve(true);
                }
            })
        })
    };

    var setenvExec = function(code, value){
        return new Promise(function(resolve, reject){
            var command = 'fw_setenv '+code;

            if(value !== null && value !== undefined){
                command += " "+value;
            }

            exec(command,
                function (error, stdout, stderr) {
                    if(error !== null){
                        reject(Error("unable-to-exec"));
                    }else{
                        resolve(true);
                    }
                }
            )
        })
    };

    return new Promise(function(resolve,reject){
        var serialized = dataToIni(data);

        var envSettings = {};

        if(data.mode === "dhcp"){
            envSettings.ipaddr = null;
            envSettings.netmask = null;
            envSettings.gatewayip = null
        }else{
            var mask = new Netmask(data.ip4).mask;
            var ipaddr = /([\d]+\.[\d]+\.[\d]+\.[\d]+)\/(\d+)/.exec(data.ip4)[1];

            envSettings.ipaddr = ipaddr;
            envSettings.netmask = mask;
            envSettings.gatewayip = data.gateway
        }

        const promises = iface === 'eth0' ? [
            setenvExec("ipaddr",envSettings.ipaddr),
            setenvExec("netmask",envSettings.netmask),
            setenvExec("gatewayip",envSettings.gatewayip),
            writeNetworkInfo(serialized)
        ] : [writeNetworkInfo(serialized)];

        if(serialized){
            Promise.all(promises)
            .then(function(res){
                exec('systemctl restart systemd-networkd',
                    function (error, stdout, stderr) {
                        if(error !== null){
                            reject(Error("unable-to-restart"));
                            //socket.emit("network:error",{code:"unable-to-set"});
                        }else{
                            resolve();
                            //socket.emit("network:ok",{code:"set"});
                        }
                    }
                )
            })
            .catch(function(error){
                console.log(error);

                reject(error)
            })
        }else{
            reject(Error("incorrect-data"));
        }
    })
};

exports.getMac = function(iface){
    return new Promise(function(resolve,reject){
        exec(`ip link show dev ${iface}`,
            function (error, stdout, stderr) {
                if(error !== null){
                    reject(Error("unable-to-get"));
                    //socket.emit("network:error",{code:"unable-to-get-mac"});
                }else{
                    var mac = /link\/ether\s([0-9a-f:]+)\s/.exec(stdout);
                    if(mac == null){
                        reject(Error("incorrect-response"));
                        //socket.emit("network:error",{code:"unable-to-get-mac"});
                    }else{
                        resolve(mac[1]);
                        //socket.emit("network:mac",{mac:mac[1]});
                    }
                }
            }
        )
    });
};

exports.setMac = function(data){
    return new Promise(function(resolve,reject) {
        if (data.operation == "reset" || (data.operation == "set" && data.ethaddr !== undefined)) {
            var args = ['fw_setenv', 'ethaddr'];
            if (data.operation == "set") {
                args = args.concat([data.ethaddr])
            }
            var escaped = shellescape(args);
            exec(escaped,
                function (error, stdout, stderr) {
                    if (error !== null) {
                        reject(Error("unable-to-set"));
                        //socket.emit("network:error", {code: "unable-to-set-mac"});
                    } else {
                        resolve();
                        //socket.emit("network:ok", {code: "mac-set"});
                    }
                }
            )
        } else {
            socket.emit("network:error", {code: "unable-to-set-mac"});
        }
    });
};

exports.getNetworkList = () => {
    return new Promise((resolve, reject) => {
        exec('ip address',
            (err, stdout, stderr) => {
                if(err === null){
                    const ifaces = {};
                    let iface = null;

                    stdout.split(/\n/).forEach(line => {
                        const parsed = /^([0-9]+): (eth[0-9]+|wlan[0-9]+):/.exec(line);
                        if(parsed !== null){
                            iface = parsed[1];
                            ifaces[iface] = {
                                name: parsed[2]
                            };
                        }else if(iface !== null){
                            const ip_parsed = /^\s+inet ([0-9.]+\/[0-9]+)/.exec(line);
                            if(ip_parsed !== null){
                                if(ifaces[iface]['ip'] === undefined){
                                    ifaces[iface]['ip'] = [];
                                }
                                ifaces[iface]['ip'].push(ip_parsed[1])
                            }
                        }
                    });

                    resolve(ifaces);
                }else{
                    reject(err);
                }
            });
    })
};
