var os = require('os');
var fs = require('fs');
var exec = require('child_process').exec;
var chartsData=[];
var chartsDataGet= function () {
    return new Promise(function(resolve,reject){
        resolve(chartsData);
    })
};
var chartsDataPush= function (data) {
    chartsData.push(data);
    if(chartsData.length>360){
         chartsData.shift()
    }
};
var getSysInfo = function(){
    return new Promise(function(resolve,reject){
        var sysinfo = {
            cpus:os.cpus(),
            freemem:os.freemem(),
            totalmem:os.totalmem(),
            uptime:os.uptime(),
            os_platform:os.platform(),
            os_release:os.release(),
            os_type:os.type(),
            hostname:os.hostname(),
            interfaces:os.networkInterfaces(),
            loadavg:os.loadavg(),
            timestamp:Date.now()
        };

        function parseKb(kb){
            return (parseInt(kb)/1024).toFixed(2);
        }
        exec("df -k",function(err,stdout){
            if(err !== null){
                reject(Error("unable-to-get-fs"))
            }else{
                try{
                    sysinfo.fs = stdout
                        .trim()
                        .split("\n")
                        .slice(1)
                        .map(function(record){
                            var fields = record.split(/\s+/);
                            return {
                                title:fields[0],
                                total:parseKb(fields[1]),
                                used:parseKb(fields[2]),
                                free:parseKb(fields[3]),
                                usedp:fields[4]}
                        })
                        .filter(function(record){
                            return record.title !== "devtmpfs" && record.title !== "tmpfs";
                        });
                    chartsDataPush(sysinfo);
                    resolve(sysinfo);
                }catch(e){
                    reject(Error("unable-to-get-fs"))
                }
            }
        })
    });
};
var getDeviceTree = function(){
    return new Promise(function(resolve,reject){
        fs.readFile("/proc/device-tree/model",function(error,data){
            if(error !== null){
                reject(Error("unable-to-get"))
            }else{
                resolve(data);
            }
        })
    })
};

// TODO: Provide all the OS info with the getReleaseInfo function

var getReleaseInfo = function(){
    return new Promise(function(resolve,reject){
        fs.readFile("/etc/os-release",function(error,data){
            if(error !== null){
                reject(Error("unable-to-get-release"))
            }else{
                try{
                    data = data
                        .toString()
                        .split("\n")
                        .slice(0,-1)
                        .map(function(record){
                            var parsed = /^([\w]+)\s*=\s*"(.+)"/.exec(record);
                            return {key:parsed[1], value:parsed[2]}
                        })
                        .filter(function(record){
                            return record.key == "PRETTY_NAME"
                        });
                    resolve(data);
                }catch(e){
                    reject(Error("unable-to-parse-release-info"))
                }
            }
        })
    })
};
var boardId=function(){
    return new Promise(function(resolve,reject){
       fs.exists('/proc/tpsid',function (exist) {
           if(exist){
               fs.readFile('/proc/tpsid','hex',function(err,file){
                   if(err){
                       reject(err)
                   }else{
                       resolve(file.toUpperCase());
                   }
               });
           }
        });
    });
};
module.exports={
    boardId:boardId,
    getSysInfo:getSysInfo,
    getDeviceTree:getDeviceTree,
    getReleaseInfo:getReleaseInfo,
    chartsDataGet:chartsDataGet
};
