/**
 * Created by stts on 14.07.2016.
 */
var dgram = require('dgram');	// UDP (datagram)
//   tftp://192.168.75.150/uImage
// Timeout for ack/data packet
var TIMEOUT = 5000; // 5 seconds
// Max retransmits if timeout
var MAX_RETRANSMITS = 3;
//packet types
var types = {
    'rrq': 1, // Read ReQuest
    'wrq': 2, // Write ReQuest
    'data': 3,
    'ack': 4,
    'err': 5,
    'oack': 6,
    // aliases:
    'read': 1,
    'write': 2
};
var errors = [
    'Not defined, see error message (if any).',
    'File not found.',
    'Access violation.',
    'Disk full or allocation exceeded.',
    'Illegal TFTP operation.',
    'Unknown transfer ID.',
    'File already exists.',
    'No such user.'
];
var tftp = module.exports = function (port, ip, opt) {
    this.ip = ip || '127.0.0.1';
    this.port = port || 69;

    // Hold the current operation
    this.current = false;
    this.readQueue = [];
    // Holds the queue of the next operations
    this.queue = [];
    this.opt = opt;
    this.fileSize;
    // Client will be created when needed
    this.client = null;
    this.lastBlock = new Number(0);
    // Display debug messages?
    this.debug = false;
};
tftp.prototype.packetFunc = function (type, msg) {
    var self = this;
    var functions = {
        1: function () {
        },
        2: function () {
        },
        3: function (msg) { //data
            var block = msg.readUInt16BE(2);
            var data = msg.slice(4);
            var expected = parseInt(self.lastBlock) + parseInt(1);
            if (block <= expected) {
                self.lastBlock = parseInt(block);
                var ack = createAck(block);
                if (block == expected) self.current.callback(null, {chunk: data, fileSize: self.fileSize});
                else if(self.debug)console.log('Re-sending ask for already seen block ' + block);
                self.send(ack);
            } else {
                console.log('Unexpected block ' + block + ' exiting');
            }
            //msg=null;
            if (self.debug) {
                console.log('> Received data. Block#: %d Bytes: %d', block, data.length);
                console.log('< Sending ACK. Block#: %d', block);
            }
            /*var ack = createAck(block);

             self.current.callback(null, {chunk:data,fileSize:self.fileSize});
             self.send(ack);*/
        },
        4: function () {
        },
        5: function () {
        },
        6: function (msg) { //oack
            var c;
            var st = 0; // variable name ==0 ;value==1
            var str = new String();
            var last_key = new String();
            var vals = {};
            for (var i = 2; i < msg.length; i++) {
                c = msg[i];
                switch (c) {
                    case 0:
                        if (st == 0) { // variable name ended
                            last_key = str;
                            st = 1;
                        }
                        else { // value ended
                            vals[last_key] = str;
                            st = 0;
                        }
                        str = '';
                        break;
                    default:
                        str += String.fromCharCode(c);
                        break;
                }

            }
            var ack = createAck(0);
            self.send(ack);
            if ('tsize' in vals) {
                self.fileSize = vals.tsize;
            }

        }
    };
    functions[type](msg);

};
tftp.prototype.createClient = function () {
    if (this.client !== null)
        return;
    var self = this;
    var message = function (msg, rinfo) {
        self.port = rinfo.port;
        if (self.current == false) {
            self.client.close();
            return;
        }
        var type = msg.readUInt16BE(0);
        self.packetFunc(type, msg);
        msg = null;
    };
    this.client = dgram.createSocket('udp4', message)

};
tftp.prototype.close = function () {
    this.current = false;
};
tftp.prototype.read = function (filename, cb) {
    if (typeof cb !== 'function') cb = function () {
    }; // Default cb to an empty function

    // Item to put into the queue
    var queueItem = {
        type: 'read',
        filename: filename,
        callback: cb,
        data: new Buffer(0) // Buffer of size 0 which will be filled up by onMessage
    };

    // Push the queueItem to the end of the queue.
    this.queue.push(queueItem);
    // Check the queue...
    this.check();
};
tftp.prototype.check = function () {
    //console.log('work check');
    if (this.current !== false)
        return;
    // If there is nothing running, and the queue is empty
    if (this.queue.length == 0) {
        // Close client
        if(this.debug) console.log('client close');
        this.client.close();
        return;
    } else {
        // Create client

        this.createClient();
    }
    this.current = this.queue.shift();
    var self = this;
    // Create the request...
    var buff = createRequest(this.current.type, this.current.filename, self.opt);
    this.send(buff, function (err, bytes) {
        if (err) {
            var err = new Error('Error when sending ' +
                self.current.type +
                ' request for' +
                self.current.filename);
            self.current.callback(err);
            self.current = false;
            self.check();
        }
    })
};
var createRequest = function (type, filename, opt, mode) {
    if (typeof type === 'string') {
        type = type.toLowerCase();
        if (types.hasOwnProperty(type)) {
            type = types[type]
        } else {
            throw 'Unknow type (' + type + ')';
        }
    }
    else if (typeof type !== 'number') {
        throw 'Unknown type (' + type + ')';
    }
    mode = mode || 'octet';
    var buffLen = 2 + filename.length + 1 + mode.length + 1;
    if (Object.keys(opt).length > 0) {
        Object.keys(opt).forEach(function (key) {
            buffLen += key.length + 1 + opt[key].toString().length + 1;
        });

    }
    var tmpBuffLen = 2 + filename.length + 1 + mode.length + 1 + 'tsize'.length + 2 + '0'.length;
    var buff = new Buffer(buffLen);
    var off = 0;
    buff.writeUInt16BE(type, 0);
    off += 2;
    buff.write(filename + '\0', off, 'ascii');
    off += filename.length + 1;
    buff.write(mode + '\0', off, 'ascii');
    off += mode.length + 1;
    if (Object.keys(opt).length > 0) {
        Object.keys(opt).forEach(function (key) {
            buff.write(key.toString() + '\0', off, 'ascii');
            off += key.length + 1;
            buff.write(opt[key].toString() + '\0', off, 'ascii');
            off += opt[key].toString().length + 1;
        });
    }

    // Return the new buffer
    return buff;
};
var createAck = function (blockNumber) {
    var type = types['ack'];
    var buff = new Buffer(4);
    buff.writeUInt16BE(type, 0);
    buff.writeUInt16BE(blockNumber, 2);
    return buff;
};
tftp.prototype.send = function (buff, cb) {
    if (typeof cb !== 'function') cb = function () {
    };
    var self = this;
    var send = function () {
        self.client.send(buff, 0, buff.length, self.port, self.ip, cb);
    };
    send()
};