var exec = require('child_process').exec;
var fs = require('fs');
var ini = require("ini");
var shellescape = require("shell-escape");

function getSystemUserList() {
    return new Promise(function (resolve, reject) {
        fs.readFile("/etc/passwd", function (error, data) {
            if (error !== null) {
                //callback(null);
                reject(Error("unable-to-read"))
            }
            else {
                var result = data.toString().split("\n")
                    .map(function (record) {
                        record = record.split(":");
                        if (record.length == 7) {
                            return {
                                "login": record[0],
                                "uid": record[2],
                                "gid": record[3],
                                "comment": record[4],
                                "home": record[5],
                                "shell": record[6]
                            };
                        } else {
                            return null;
                        }
                    })
                    .filter(function (record) {
                        return record !== null;
                    });
                if (result.length > 0) {
                    resolve(result);
                } else {
                    reject(Error("incorrect-output"));
                }
            }
        })
    })
}

exports.getSystemUserList = getSystemUserList;

exports.usersSystemSet = function (data) {
    return new Promise(function (resolve, reject) {
        if (data.operation == "add" || data.operation == "update") {
            if (/[a-zA-Z0-9_\-]{6,}/.test(data.password) &&
                /^[_A-Za-z0-9][-@_\.A-Za-z0-9]{3,}$/.test(data.username) && !/^systemd/.test(data.username)
            ) {
                var genPassCommand = "/usr/bin/python3 -c 'import crypt; print( crypt.crypt(\"" + data.password + "\",\"\$6\$4jZ2FV34JZQ01jT\$\"))'";

                exec(genPassCommand, function (error, stdout) {
                    if (error !== null || stdout == null) {
                        reject(Error("unable-to-gen-password"));
                    } else {
                        getSystemUserList()
                            .then(function (result) {
                                var command;
                                var allowed = result
                                    .filter(function (row) {
                                        return /^\/home\/(.+)$/.test(row.home)
                                            && !/^systemd-/.test(row.login);
                                    })
                                    .map(function (row) {
                                        return row.login
                                    });

                                if (data.operation == "add") {
                                    if (allowed.indexOf(data.username) !== -1) {
                                        reject(Error("incorrect-input"));
                                    } else {
                                        command = "/usr/sbin/useradd -p " + stdout.replace(/\n/, "").replace(/\$/g, "\\$") + " " + data.username;
                                        exec(command,
                                            function (error, stdout, stderr) {
                                                if (error !== null) {
                                                    reject(Error("unable-to-add"));
                                                } else {
                                                    resolve("add");
                                                }
                                            }
                                        )
                                    }
                                } else if (data.operation == "update") {
                                    if (allowed.indexOf(data.username) == -1) {
                                        reject(Error("incorrect-input"));
                                    } else {
                                        command = "/usr/sbin/usermod -p " + stdout.replace(/\n/, "").replace(/\$/g, "\\$") + " " + data.username;
                                        exec(command,
                                            function (error, stdout, stderr) {
                                                if (error !== null) {
                                                    reject(Error("unable-to-update"));
                                                } else {
                                                    resolve("update");
                                                }
                                            }
                                        )
                                    }
                                }
                            })
                            .catch(function () {
                                reject(Error("unable-to-get"))
                            })
                    }
                });
            } else {
                reject(Error("incorrect-input"));
            }
        } else if (data.operation == "delete") {
            if (!/^systemd/.test(data.username) && data.username != "root" && /^[_A-Za-z0-9][-@_\.A-Za-z0-9]{3,}$/.test(data.username)) {
                var command = "/usr/sbin/userdel -r " + data.username;
                exec(command,
                    function (error, stdout, stderr) {
                        if (error !== null) {
                            reject(Error("unable-to-delete"));
                        } else {
                            resolve("delete");
                        }
                    }
                )
            } else {
                reject(Error("incorrect-input"));
            }
        } else {
            reject(Error("incorrect-input"));
        }
    })
};

exports.usersWebSet = function (socket, data) {
    return new Promise(function (resolve, reject) {
        if (data.new == data.repeat) {
            exec("openssl passwd -apr1 " + data.new, function (error, stdout) {
                if (error !== null || stdout == null) {
                    reject("unable-to-set");
                    //socket.emit("users:web:error", {code: "unable-to-set"});
                } else {
                    var content = "admin:" + stdout.replace(/\n/, "") + ":no comment";
                    fs.writeFile("/var/www/tps-wan/conf/.htpasswd", content, function (error) {
                        if (error) {
                            reject("unable-to-set");
                            //socket.emit("users:web:error", {code: "unable-to-set"});
                        }
                        else {
                            resolve();
                            socket.emit("users:web:ok", {code: "set"});
                        }
                    })
                }
            });
        } else {
            reject("unable-to-set");
            //socket.emit("users:web:error", {code: "unable-to-set"});
        }
    })
};
